#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

int getIntValue(GtkWindow *window, char *widgetname)
{
   GtkWidget *target =lookup_widget (window, widgetname);
   return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (target));
}

char *getStringValue(GtkWindow *window, char *widgetname)
{
   GtkWidget *target =lookup_widget (window, widgetname);
   return  (gchar *)gtk_entry_get_text (GTK_ENTRY (target));
}

void setStringValue(GtkWindow *window, char *widgetname,char *string)
{
   GtkWidget *target =lookup_widget (window, widgetname);
   gtk_entry_set_text (GTK_ENTRY (target), string);
}

int executeQuery(SQL *sock,char *query)
{   
   writeLog("Executing: %s\n",query);
   if (sql_query(sock,query) != 0)
   {
      showSqlError(sock,"Query failed");
      return -1;
   }
   return 0;
}

int getNumRecords(Params *p)
{
   SQL_RES *res;
   char query[MAXQUERY];
   
   ldebug(2,"getNumRecords called\n");
   if(p->countflag == NOCOUNT)
     return 0;
   
   snprintf(query,sizeof(query) - 1,SELECT,p->table);
   
   if (executeQuery(p->dbsock, query) != 0)
     return 0;
   
   res = sql_store_result(p->dbsock);
   if(res == NULL)
   {
      showSqlError(p->dbsock, "getNumRecords: Cannot store result");
      return 0;
   }

   p->numrecs = sql_num_rows(res);
   
   sql_free_result(res);
   return p->numrecs;
}

void setGUICounters(Params *p, int nrFields, int nrRecords)
{
   char string[10]; /* should be enough ;-) */
   
   ldebug(2,"setGUICounters called\n");
   snprintf(string,sizeof(string) - 1,"%d",nrFields);
   setStringValue(p->mainwindow, "fieldcount", string);
   
   snprintf(string,sizeof(string) - 1,"%d",nrRecords);
   setStringValue(p->mainwindow, "recordcount", string);  
}

void setSortKey(int row)
{
   Params *p = getParams();
   p->sortkey = row;
   refreshFieldTab();
}

void newTab(TabInfo *ti, void *callback, char *refname)
{
   GtkWidget *tab;
   
   ldebug(2,"newTab entered\n");
   
   tab = GTK_WIDGET(gtk_clist_new_with_titles(ti->nrcols, ti->titles));

   gtk_signal_connect(GTK_OBJECT(tab), "select_row",
		      GTK_SIGNAL_FUNC(callback), (gpointer) ti);
   
   gtk_widget_ref (tab);
   gtk_object_set_data_full (GTK_OBJECT(ti->mainwindow), refname, tab,
			     (GtkDestroyNotify) gtk_widget_unref);
   
   gtk_widget_show (tab);
  
   gtk_container_add(GTK_CONTAINER(ti->frame), tab);
   
   ti->clist = tab;
}

void destroyTab(TabInfo *ti)
{
   ldebug(2,"destroyTab entered\n");
   if(ti == NULL || ti->clist == NULL) 
     return;
   
   if(! GTK_IS_WIDGET(ti->clist))
   {
      ti->clist = NULL;
      return;
   }
   
   gtk_widget_hide(ti->clist);
   gtk_widget_destroy(ti->clist);

   destroyTabInfostruct(ti);
}

void destroyAllTabs(Params *p)
{
   int x;
   
   for(x=0;x<p->nrtabs;x++)
   {
      destroyTab(p->tabs[x]);
      p->tabs[x] = NULL;
   }
}

void refreshRecordTab()
{
   Params *p=getParams();

   GtkCheckMenuItem *auto_refresh =
     (GTK_CHECK_MENU_ITEM(lookup_widget (get_top_window(),
					 "settings_auto_refresh")));

   if(auto_refresh != NULL && auto_refresh->active != 0)
     showRecords(p);
}

void refreshFieldTab()
{
   Params *p=getParams();
   showFields(p);
}

GtkStyle *newFontStyle(GtkStyle *source, gchar *fontname)
{
   GtkStyle *style= gtk_style_copy(source);

#ifdef HASNOFONTSTYLE
;
#else
   gdk_font_unref (style->font);
   style->font = gdk_font_load(fontname);
#endif

   return style;
}

GtkStyle *newColorStyle(GtkStyle *source, double rgb[3])
{
   GtkStyle *style= gtk_style_copy(source);
   
   GdkColor *c = (GdkColor*) g_malloc(sizeof(GdkColor));
   c->red   = (int)(rgb[0]*65535);
   c->green = (int)(rgb[1]*65535);
   c->blue  = (int)(rgb[2]*65535);
   
   style->fg[GTK_STATE_NORMAL] = *c;
   
   return style;
}

GtkStyle *newColorFontStyle(GtkStyle *source, double rgb[3], gchar *fontname)
{
   GtkStyle *style= newColorStyle(source, rgb);
   
#ifdef HASNOFONTSTYLE
   ;
#else
   gdk_font_unref (style->font);
   style->font = gdk_font_load(fontname);
#endif
   
   return style;
}
