#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* If we arrive here it means the user has selected a row in the list. */
void record_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   TabInfo *ti = (TabInfo *)data;

   ldebug(2,"record_selected entered\n");

   ti->selectedrow = row; /* single click : just preserve selection */
   ldebug(3,"You selected row %d.\n ",row);
   
   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"record_selected: doubleclick detected\n");
      
      if(ti->table == NULL)
      {
	 showMessage("Currently no table selected !");
	 return;
      }
      
      ldebug(3,"You doubleclicked row %d. More specifically you clicked in "
	     "column %d\n\n", row, column);
      
      newModifyWindow(ti);
   }
}

void setSortKeyAction(GtkWidget *w, gpointer user_data)
{  
   ldebug(2,"setSortKeyAction entered\n");
}

/* This should work for all available tabs (clists) */
void setTabColumnWidths(TabInfo *ti, int type)
{
   int x;
   int cwidth;
   
   ldebug(2,"setTabColumnWidths entered\n");
   
   /* set column sizes according to the fieldlength : */
   for(x=0;x<ti->nrcols;x++)
   {
      cwidth = (strlen(ti->titles[x]));
      switch(type)
      {
       case RECORDSTAB: 
       case SEARCHTAB :
       case QUERYTAB  :
	 if ( ti->fields[x].length > cwidth )
	   cwidth = ti->fields[x].length;
	 break;
       case FIELDSTAB :
	 cwidth += 8; /* slightly larger */
	 break;
      }
      
      cwidth *= FONTWIDTH;
      ldebug(3,"Column width for column %d : %d\n",x,cwidth);
      gtk_clist_set_column_width(GTK_CLIST(ti->clist), x, cwidth);
   }
}

/* This should work for all available tabs (clists) */
void saveTabAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   TabInfo *ti = (TabInfo *) user_data;
   int x,y;
   
   ldebug(2,"saveTabAction entered\n");
   
   if(ti == NULL || ti->table == NULL)
   {
      showMessage("Cannot save records:\n no records in tab !");
      return;
   }
   
   output = fopen(ti->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save records:\n open file %s failed !",
			ti->selectedname);
      return;
   }
   
   for(y=0;y<ti->nrrows; y++)
   {
      for(x=0;x<ti->nrcols;x++)
      {  
	 gchar *content;
	 
	 gtk_clist_get_text(GTK_CLIST(ti->clist), y, x, &content);
	 fprintf(output,"%s%s",content,DELIMITER);
	 ldebug(3,"%s%s",content,DELIMITER);
      }
      fprintf(output,"\n");
      ldebug(3,"\n");
   }
   
   fclose(output);
}

void showRecords(Params *p)
{
   int     x;
   SQL_ROW row;
   SQL_RES *res;
   char    query[MAXQUERY];
   TabInfo *ti;
   
   ldebug(2,"showRecords entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show records:\n select a table first !");
      return;
   }

   /* p->sortkey is always defined if table != NULL */
   snprintf(query,sizeof(query) - 1,SELECT ORDERBY, p->table);
   strcat(query,p->fields[p->sortkey].name);
   
   if (executeQuery(p->dbsock, query))
   {
      showSqlError(p->dbsock,"Query in showRecords failed !");
      return;
   }
   
   destroyTab(p->tabs[RECORDSTAB]);
   /* destroy possible previous RecordTab + TabInfo */

   res = sql_store_result(p->dbsock);
   ti  = newTabInfostruct(p, RECORDSTAB, 
			  sql_num_fields(res), sql_num_rows(res));
   
   /* Now fetch the fieldnames for this selection : */
   for(x=0;x<ti->numflds;x++)
     ti->titles[x] = ti->fields[x].name;
   
   newTab(ti,record_selected,"recordslist");
   setTabColumnWidths(ti,RECORDSTAB);
   
   /*  Change the sort key if the column title is clicked : */
   /* This doesn't work yet, why not ? */
/*   gtk_signal_connect(GTK_OBJECT(ti->clist), "click-column",
		      GTK_SIGNAL_FUNC(setSortKeyAction),(gpointer) ti);
*/   
   gtk_clist_freeze(GTK_CLIST(ti->clist));
   
   for(x=0;x < ti->nrrows;x++)
   {
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(ti->dbsock,"Error reading record %d !",x);
	 break;
      }
      
      /* write a row to the tab : */
      gtk_clist_append(GTK_CLIST(ti->clist),row);
   }
   
   gtk_clist_thaw(GTK_CLIST(ti->clist));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), RECORDSTAB);

   p->tabs[RECORDSTAB] = ti;
}
