#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

static GtkWindow *dialogbox;
static GtkWidget *logwindow;
static GtkTextBuffer *logbuffer;

void debug(char *format, ...)
{
#ifdef DEBUG
   va_list ap;
   
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

void ldebug(int level, char *format, ...)
{
#ifdef DEBUG
   va_list ap;
   
   if (DEBUGLEVEL < level)
     return;
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

void
on_dialogcancel_clicked (GtkButton *button, gpointer user_data)
{
   ;
}

void
on_dialogclose_clicked (GtkButton *button,gpointer user_data)
{
   ;
}

void on_dialogok_clicked (GtkButton *button, gpointer user_data)
{
   ;
}

void
on_dialogyes_clicked (GtkButton *button, gpointer user_data)
{
   ;
}

void
on_dialogno_clicked (GtkButton *button, gpointer user_data)
{
   ;
}

void defaultAction(GtkButton *button, gpointer user_data)
{
   if(dialogbox == NULL)
     return;
   gtk_widget_hide(GTK_WIDGET(dialogbox));
   gtk_widget_destroy(GTK_WIDGET(dialogbox));
}

void showMessageBox(char *text)
{
   GtkWidget *button;
   
   dialogbox = create_dialogbox();
   
   /* Turn on the buttons we want here : */
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogclose")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogcancel")));
    gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogok")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogyes")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogno")));
   
   gtk_widget_show(GTK_WIDGET(dialogbox));
   
   if(text != NULL)
     gtk_label_set_text
     (GTK_LABEL(lookup_widget(dialogbox, "dialogtext")), text);
   
   button =lookup_widget (dialogbox, "dialogok");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (defaultAction), NULL);
}

/* change into showRecordConfirmBox ? */
void showConfirmBox(char *text, gpointer yesAction,
		    gpointer noAction, gpointer rec)
{
   Params *p = getParams();
   GtkWidget *button = NULL;
   GtkCheckMenuItem *confirm_actions;
   void (*ptr2Action)(GtkButton *, gpointer) = yesAction; /* function pointer */
   
   if(yesAction == NULL)
     yesAction = defaultAction;
   if(noAction == NULL)
     noAction = defaultAction;
   
   dialogbox = create_dialogbox();
   
   /* Turn on the buttons we want here : */
// gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogclose")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogcancel")));
// gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogok")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogyes")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogno")));
   
/* The skip confirmation test must be done AFTER the creation of the dialog_box to 
   avoid the yesAction having a NULL pointer for the button. Deletion of
   the (invisible) dialogbox then takes place in the usual way in the
   yesAction */
   
   confirm_actions = 
     (GTK_CHECK_MENU_ITEM(lookup_widget (p->mainwindow,
					 "settings_confirm_actions")));
   if(confirm_actions != NULL && confirm_actions->active == 0)
   {
      /* do not popup a dialogbox */
      button =lookup_widget (dialogbox, "dialogyes");
      ptr2Action(GTK_BUTTON(button),rec);
      return;
   }
   
   gtk_widget_show(GTK_WIDGET(dialogbox));
   
   if(text != NULL)
     gtk_label_set_text
     (GTK_LABEL(lookup_widget(dialogbox, "dialogtext")), text);
   
   button =lookup_widget (dialogbox, "dialogcancel");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (defaultAction), NULL);
   
   button =lookup_widget (dialogbox, "dialogyes");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (yesAction), rec);

   button =lookup_widget (dialogbox, "dialogno");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (noAction), rec);
}

void showVarArgMessage(char *s,...)
{
   va_list ap;
   char message[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(message,sizeof(message) - 1, s, ap);
   ldebug(2,"Message: %s\n",message);
   
   showMessageBox(message);
}

void showMessage(char *message)
{
   ldebug(2,"Message: %s\n",message);
   
   showMessageBox(message);
}

/* allows for writeLog calls without having to supply the logwindow widget: */
void setLogWindow(void)
{
   Params *p = getParams();
   logwindow = p->logwindow;
   logbuffer = p->logbuffer;
}

void showSqlError(SQL *sock, char *s,...)
{
   va_list ap;
   char error[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(error,sizeof(error) - 1, s, ap);
   
   writeLog("Error: %s\n",error);
   writeLog("Error: %s\n",sql_error(sock));
}

/* This writes to the logbrowser */
void writeLog(char *s,...)
{
   va_list ap;
   char logline[MAXMESSAGE];
   GtkTextIter    txtend;
   
   va_start(ap,s);
   vsnprintf(logline,sizeof(logline) - 1, s, ap);
   ldebug(2,"%s\n",logline);

   gtk_text_buffer_get_end_iter (logbuffer, &txtend);
   gtk_text_buffer_insert (logbuffer, &txtend, logline, -1);
   
   /* Always scroll to (new) end of logwindow : */
   gtk_text_buffer_get_end_iter (logbuffer, &txtend);
   gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (logwindow),
				 &txtend, 0.0, TRUE, 0.0, 1.0);
}

void saveLogAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   GtkTextBuffer *logbuffer;
   GtkTextIter    txtstart;
   GtkTextIter    txtend;
   
   Params *p = getParams();
      
   ldebug(2,"saveLogAction entered\n");
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save log:\n open file %s failed !",
		  p->selectedname);
      return;
   }

   logbuffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW( logwindow ));
   gtk_text_buffer_get_start_iter (logbuffer, &txtstart);
   gtk_text_buffer_get_end_iter   (logbuffer, &txtend);
   fprintf(output,
	   gtk_text_buffer_get_text(logbuffer,&txtstart, &txtend, FALSE ));
   fclose(output);

   writeLog("Log saved to %s\n",p->selectedname);
}

