#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

int getIntValue(GtkWindow *window, char *widgetname)
{
   GtkWidget *target =lookup_widget (window, widgetname);
   return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (target));
}

char *getStringValue(GtkWindow *window, char *widgetname)
{
   GtkWidget *target =lookup_widget (window, widgetname);
   return  (gchar *)gtk_entry_get_text (GTK_ENTRY (target));
}

void setStringValue(GtkWindow *window, char *widgetname,char *string)
{
   GtkWidget *target =lookup_widget (window, widgetname);
   gtk_entry_set_text (GTK_ENTRY (target), string);
}

int executeQuery(SQL *sock,char *query)
{   
   writeLog("Executing: %s\n",query);
   if (sql_query(sock,query) != 0)
   {
      showSqlError(sock,"Query failed");
      return -1;
   }
   return 0;
}

/* Routine to copy all necessary field data, doing all the mallocs etc. as well */
SQL_FIELD *copyFields(SQL_FIELD *src, int numflds)
{
   int y;
   SQL_FIELD *tgt;
   
   ldebug(2,"copyFields called\n");
   
   if(src == NULL)
      return NULL;
   
   /* Malloc space and copy all normal fields */
   tgt = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * numflds);
   memcpy(tgt, src, sizeof(SQL_FIELD) * numflds); 

   for(y=0; y<numflds; y++)
   {
      int tlen;
      /* Copy pointer "name" and "def" as well. Note that we do not copy other strings 
       like "table" here, so fieldinfo is incomplete ! */
      
      ldebug(4,"copyFields: now copying %d : %s\n",y, src[y].name);
      
      tlen        = strlen(src[y].name) + 1;
      tgt[y].name = (char *)g_malloc(tlen);
      strncpy(tgt[y].name, src[y].name, tlen);
      
      ldebug(4,"copyFields: now copying %d : %s\n",y, src[y].def);
      
      if(src[y].def != NULL)
      {
	 tlen        = strlen(src[y].def) + 1;
	 tgt[y].def  = (char *)g_malloc(tlen);
	 strncpy(tgt[y].def, src[y].def, tlen);
      }
   }
   
   return tgt;
}

/* Routine to free all mallocated memory space in copyFields above */
void freeFields(SQL_FIELD *tgt, int numflds)
{
   int y;
   
   ldebug(2,"freeFields called\n");
   
   if(tgt == NULL)
     return;
   
   for(y=0; y<numflds; y++)
   {
      g_free(tgt[y].name);
      if(tgt[y].def != NULL)
	g_free(tgt[y].def);
   }
   
   g_free(tgt);
}

int getNumRecords(void)
{
   SQL_RES *res;
   char query[MAXQUERY];
   Params *p = getParams();
   
   ldebug(2,"getNumRecords called\n");
   if(p->countflag == NOCOUNT)
     return 0;
   
   snprintf(query,sizeof(query) - 1,SELECT,p->table);
   
   if (executeQuery(p->dbsock, query) != 0)
     return 0;
   
   res = sql_store_result(p->dbsock);
   if(res == NULL)
   {
      showSqlError(p->dbsock, "getNumRecords: Cannot store result");
      return 0;
   }

   p->numrecs = sql_num_rows(res);
   
   sql_free_result(res);
   return p->numrecs;
}

void setGUICounters(int nrFields, int nrRecords)
{
   char string[10]; /* should be enough ;-) */
   
   ldebug(2,"setGUICounters called\n");
   snprintf(string,sizeof(string) - 1,"%d",nrFields);
   setStringValue(getTopWindow(), "fieldcount", string);
   
   snprintf(string,sizeof(string) - 1,"%d",nrRecords);
   setStringValue(getTopWindow(), "recordcount", string);  
}

/* currently not used : */
GtkStyle *newColorStyle(GtkStyle *source, double rgb[3])
{
   GtkStyle *style= gtk_style_copy(source);
   
   // allocate a new color that survives this function call :
   GdkColor *c = (GdkColor*) g_malloc(sizeof(GdkColor)); 
   c->red   = (int)(rgb[0]*65535);
   c->green = (int)(rgb[1]*65535);
   c->blue  = (int)(rgb[2]*65535);
   
   style->fg[GTK_STATE_NORMAL] = *c;
   
   return style;
}

void setBoldRedFont(GtkWidget *widget)
{
   PangoFontDescription *font_desc;
   double rgb[3] =  REDCOLOR ;
   
   GdkColor c ;
   c.red   = (int)(rgb[0]*65535);
   c.green = (int)(rgb[1]*65535);
   c.blue  = (int)(rgb[2]*65535);
   
   font_desc = pango_font_description_from_string (BOLDFONT);
   gtk_widget_modify_fg (widget, GTK_STATE_NORMAL, &c);
   gtk_widget_modify_font (widget, font_desc);

   pango_font_description_free (font_desc);
}

void setBoldFont(GtkWidget *widget)
{
   PangoFontDescription *font_desc;
   
   font_desc = pango_font_description_from_string (BOLDFONT);
   gtk_widget_modify_font (widget, font_desc);
   
   pango_font_description_free (font_desc);
}

